#include <PSX/File/Processor/cd_file_processor.hpp>
#include <CD/cd_internal.hpp>
#include <stdio.h>

namespace JabyEngine {
    void CDFileProcessor :: start_cur_job() {
        using CD::internal::FileInfo;
        using CD::internal::SectorBufferAllocator;

        const auto& cur_job = *this->jobs.files;
        const auto& cur_lba = this->lba[cur_job.rel_lba_idx];

        CD::internal::read_file(FileInfo::from(cur_lba), SectorBufferAllocator::create(this, [](void* ctx) -> CD_IO::DataSector* {
            CDFileProcessor &self = *reinterpret_cast<CDFileProcessor*>(ctx);

            return self.circular_buffer.allocate();
        }));

        printf(">>> CD needs to load LBA: %i -> %i\n", cur_lba.lba, cur_lba.size_words);
    }

    void CDFileProcessor :: setup(const volatile AutoLBAEntry* lba, JobArray jobs, uint8_t* work_area) {
        this->lba       = const_cast<const AutoLBAEntry*>(lba);
        this->work_area = (work_area + this->circular_buffer.setup(reinterpret_cast<CD_IO::DataSector*>(work_area)));
        this->jobs      = jobs;

        CDFileProcessor::start_cur_job();
    }

    Progress CDFileProcessor :: process() {
        switch(CD::internal::read_current_state()) {
            case CD::internal::State::Done:
                // Need to start next job? 
                // Does the user trigger this?
                printf("Done: %i\n", this->circular_buffer.has_data());
                return Progress::Done;

            case CD::internal::State::BufferFull:
                // We have to process data and unpause the CD drive
                // Error case for now
                return Progress::Error;

            case CD::internal::State::Reading:
                // Do we have data? Use it!
                return Progress::InProgress;

            case CD::internal::State::Error:
                // Error for real!
                return Progress::Error;
        }

        return Progress::Error;
    }
}